@@ -1,16 +1,15 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# Copyright (c) Mito.
-# Distributed under the terms of the Modified BSD License.
-
+# Copyright (c) Saga Inc.
+# Distributed under the terms of the GPL License.
 """
 Contains helpful utility functions
 """
 import json
 import re
 import uuid
-from typing import Any, Dict, List, Optional, Set
+from typing import Any, Dict, List, Optional, Set, Tuple
 from mitosheet.types import ColumnHeader, ColumnID
 
 import numpy as np
@@ -22,7 +21,7 @@ from mitosheet.sheet_functions.types.utils import get_float_dt_td_columns
 # We only send the first 1500 rows of a dataframe; note that this
 # must match this variable defined on the front-end
 MAX_ROWS = 1_500
-
+MAX_COLUMNS = 1_500
 
 def get_first_unused_dataframe_name(existing_df_names: List[str], new_dataframe_name: str) -> str:
     """
@@ -44,7 +43,8 @@ def get_first_unused_dataframe_name(existing_df_names: List[str], new_dataframe_
 
 def get_valid_dataframe_name(existing_df_names: List[str], original_dataframe_name: str) -> str:
     """
-    Given a string, turns it into a valid dataframe name.
+    Given a string, turns it into a valid dataframe name, making sure to 
+    not overlap with any existing dataframe names.
     """
     # We get all the words from the original name, and append them with underscores
     dataframe_name = '_'.join(filter(None, [
@@ -75,6 +75,13 @@ def get_valid_dataframe_names(existing_df_names: List[str], original_df_names: L
     
     return final_names
 
+def is_default_df_names(df_names: List[str]) -> bool:
+    """
+    Returns true if the df names list is the default df names
+    created when the widget state container is initialized
+    """
+    return len(df_names) > 0 and df_names == [f'df{i + 1}' for i in range(len(df_names))]
+
 def dfs_to_array_for_json(
         modified_sheet_indexes: Set[int],
         previous_array: List,
@@ -83,7 +90,6 @@ def dfs_to_array_for_json(
         df_sources: List[str],
         column_spreadsheet_code_array: List[Dict[ColumnID, str]],
         column_filters_array: List[Dict[ColumnID, Any]],
-        column_mito_types_array: List[Dict[ColumnID, str]],
         column_ids: ColumnIDMap,
         column_format_types: List[Dict[ColumnID, Dict[str, str]]]
     ) -> List:
@@ -98,11 +104,11 @@ def dfs_to_array_for_json(
                     df_sources[sheet_index],
                     column_spreadsheet_code_array[sheet_index],
                     column_filters_array[sheet_index],
-                    column_mito_types_array[sheet_index],
                     column_ids.column_header_to_column_id[sheet_index],
                     column_format_types[sheet_index],
-                    # We only send the first 1500 rows, and this must 
+                    # We only send the first 1500 rows and 1500 columns
                     max_length=MAX_ROWS,
+                    max_columns=MAX_COLUMNS
                 ) 
             )
         else:
@@ -117,10 +123,10 @@ def df_to_json_dumpsable(
         df_source: str,
         column_spreadsheet_code: Dict[ColumnID, str],
         column_filters: Dict[ColumnID, Any],
-        column_mito_types: Dict[ColumnID, str],
         column_headers_to_column_ids: Dict[ColumnHeader, ColumnID],
         column_format_types: Dict[ColumnID, Dict[ColumnID, str]],
-        max_length: Optional[int]=MAX_ROWS, # How many items you want to display
+        max_length: Optional[int]=MAX_ROWS, # How many items you want to display. None when using this function to get unique value counts
+        max_columns: int=MAX_COLUMNS # How many columns you want to display. Unlike max_length, this is always defined
     ) -> Dict[str, Any]:
     """
     Returns a dataframe represented in a way that can be turned into a 
@@ -141,7 +147,7 @@ def df_to_json_dumpsable(
         columnIDsMap: ColumnIDsMap;
         columnSpreadsheetCodeMap: Record<string, string>;
         columnFiltersMap: ColumnFilterMap;
-        columnMitoTypeMap: ColumnMitoTypeMap;
+        columnnDtypeMap: Record<ColumnID, string>;
         index: (string | number)[];
         columnFormatTypeObjMap: ColumnFormatTypeObjMap;
     }
@@ -155,6 +161,9 @@ def df_to_json_dumpsable(
         # we only show the first max_length rows!
         df = original_df.head(n=max_length if max_length else num_rows).copy(deep=True)
 
+    # we only show the first max_columns columns!
+    df = df.iloc[: , :max_columns]
+
     float_columns, date_columns, timedelta_columns = get_float_dt_td_columns(df)
     # Second, we figure out which of the columns contain dates, and we
     # convert them to string columns (for formatting reasons).
@@ -184,6 +193,7 @@ def df_to_json_dumpsable(
                 d[idx] = 'NaN'
 
     final_data = []
+    column_dtype_map = {}
     for column_index, column_header in enumerate(json_obj['columns']):
         # Because turning the headers to json takes multi-index columns and converts
         # them into lists, we need to turn them back to tuples so we can index into the
@@ -191,12 +201,15 @@ def df_to_json_dumpsable(
         if isinstance(column_header, list):
             column_header = tuple(column_header)
 
+        column_id = column_headers_to_column_ids[column_header]
+
         column_final_data = {
-            'columnID': column_headers_to_column_ids[column_header],
+            'columnID': column_id,
             'columnHeader': column_header,
             'columnDtype': str(original_df[column_header].dtype),
             'columnData': []
         }
+        column_dtype_map[column_id] = str(original_df[column_header].dtype)
         for row in json_obj['data']:
             column_final_data['columnData'].append(row[column_index])
         
@@ -216,7 +229,7 @@ def df_to_json_dumpsable(
         },
         'columnSpreadsheetCodeMap': column_spreadsheet_code,
         'columnFiltersMap': column_filters,
-        'columnMitoTypeMap': column_mito_types,
+        'columnDtypeMap': column_dtype_map,
         'index': json_obj['index'],
         'columnFormatTypeObjMap': column_format_types
     }
@@ -233,3 +246,28 @@ def get_new_id() -> str:
     return str(uuid.uuid4())
 
 
+def run_command(command_array: List[str]) -> Tuple[str, str]:
+    """
+    An internal command that should be used to run all commands
+    that run on the command line, so that output from failing
+    commands can be captured.
+    """
+    import subprocess
+    completed_process = subprocess.run(
+        command_array, 
+        # NOTE: we do not use the capture_output variable, as this doesn't work before
+        # python 3.7
+        stdout=subprocess.PIPE, 
+        stderr=subprocess.STDOUT,
+        # NOTE: we use universal_newlines to get the result back as text, 
+        # but we don't use text=True, because we want to work before 3.7 when
+        # text was introduced. See here: https://stackoverflow.com/questions/41171791/how-to-suppress-or-capture-the-output-of-subprocess-run
+        universal_newlines=True
+    )
+    # We default the stdout and stderr to empty strings if they are not strings, 
+    # to make code that handles them have an easier time (they might be None)
+    stdout = completed_process.stdout if isinstance(completed_process.stdout, str) else ''
+    stderr = completed_process.stderr if isinstance(completed_process.stderr, str) else ''
+    return stdout, stderr
+
+
